"""
HTTP response code tests for Catzilla framework

Tests 405 Method Not Allowed, 415 Unsupported Media Type,
and other HTTP response scenarios with the new routing system.
"""

import pytest
import json
from unittest.mock import Mock, patch
from catzilla import App, Request, Response, JSONResponse
from catzilla.types import Request as RequestType


class TestHTTPResponseCodes:
    """Test HTTP response codes with advanced routing"""

    def test_405_method_not_allowed(self):
        """Test 405 Method Not Allowed responses"""
        app = App()

        @app.get("/api/data")
        def get_data(request):
            return JSONResponse({"data": "value"})

        @app.post("/api/data")
        def post_data(request):
            return JSONResponse({"message": "created"})

        # Test that the route exists for GET and POST
        routes = app.router.routes()
        methods = {r["method"] for r in routes if r["path"] == "/api/data"}
        assert "GET" in methods
        assert "POST" in methods

        # Test 405 detection in router
        route, params, allowed_methods = app.router.match("PUT", "/api/data")
        assert route is None  # No route for PUT
        assert allowed_methods == {"GET", "POST"}  # But these methods are allowed

    def test_404_not_found(self):
        """Test 404 Not Found responses"""
        app = App()

        @app.get("/existing")
        def existing_route(request):
            return JSONResponse({"exists": True})

        # Test that non-existent routes return None
        route, params, allowed_methods = app.router.match("GET", "/nonexistent")
        assert route is None
        assert params == {}
        assert allowed_methods is None  # Path doesn't exist at all

    def test_415_unsupported_media_type_detection(self):
        """Test 415 Unsupported Media Type scenarios"""
        # This tests the concept - actual 415 handling is in C code

        # Test cases where 415 should be returned
        unsupported_content_types = [
            "application/xml",
            "text/xml",
            "application/x-protobuf",
            "invalid/content-type"
        ]

        supported_content_types = [
            "application/json",
            "application/x-www-form-urlencoded",
            "multipart/form-data",
            "text/plain"
        ]

        # Test that we can identify unsupported types
        # (This would normally be done in C code)
        for content_type in unsupported_content_types:
            # Simulated check for unsupported content type
            is_supported = content_type in [
                "application/json",
                "application/x-www-form-urlencoded",
                "multipart/form-data",
                "text/plain"
            ]
            assert not is_supported, f"Content type {content_type} should not be supported"

        for content_type in supported_content_types:
            is_supported = content_type in [
                "application/json",
                "application/x-www-form-urlencoded",
                "multipart/form-data",
                "text/plain"
            ]
            assert is_supported, f"Content type {content_type} should be supported"


class TestRequestHandling:
    """Test request handling with path parameters"""

    def test_path_parameter_injection(self):
        """Test that path parameters are correctly injected into requests"""
        # This simulates what happens in the C code

        # Create a mock request with path parameters
        request = Mock(spec=RequestType)
        request.path_params = {"user_id": "123", "post_id": "456"}
        request.has_path_params = True
        request.path_param_count = 2

        # Test that parameters are accessible
        assert request.path_params["user_id"] == "123"
        assert request.path_params["post_id"] == "456"
        assert request.has_path_params is True

    def test_request_without_path_params(self):
        """Test requests without path parameters"""
        request = Mock(spec=RequestType)
        request.path_params = {}
        request.has_path_params = False
        request.path_param_count = 0

        assert request.path_params == {}
        assert request.has_path_params is False

    def test_path_param_access_patterns(self):
        """Test different ways to access path parameters"""
        request = Mock(spec=RequestType)
        request.path_params = {"id": "123", "name": "test"}

        # Test direct access
        assert request.path_params["id"] == "123"

        # Test get with default
        assert request.path_params.get("id") == "123"
        assert request.path_params.get("missing", "default") == "default"


class TestComplexRoutingScenarios:
    """Test complex routing scenarios and edge cases"""

    def test_nested_resource_routing(self):
        """Test deeply nested resource routes"""
        app = App()

        @app.get("/api/v1/companies/{company_id}/departments/{dept_id}/employees/{emp_id}")
        def get_employee(request):
            return JSONResponse({
                "company_id": request.path_params.get("company_id"),
                "dept_id": request.path_params.get("dept_id"),
                "emp_id": request.path_params.get("emp_id")
            })

        @app.post("/api/v1/companies/{company_id}/departments/{dept_id}/employees")
        def create_employee(request):
            return JSONResponse({
                "company_id": request.path_params.get("company_id"),
                "dept_id": request.path_params.get("dept_id"),
                "action": "create"
            })

        routes = app.router.routes()
        assert len(routes) == 2

        # Check that routes are registered correctly
        paths = {r["path"] for r in routes}
        assert "/api/v1/companies/{company_id}/departments/{dept_id}/employees/{emp_id}" in paths
        assert "/api/v1/companies/{company_id}/departments/{dept_id}/employees" in paths

    def test_route_precedence(self):
        """Test route precedence and matching order"""
        app = App()

        @app.get("/api/users/profile")  # Specific route
        def get_profile(request):
            return JSONResponse({"type": "profile"})

        @app.get("/api/users/{user_id}")  # General route
        def get_user(request):
            return JSONResponse({"type": "user", "id": request.path_params.get("user_id")})

        # Test that both routes are registered
        routes = app.router.routes()
        assert len(routes) == 2

        # Test matching behavior
        profile_route, profile_params, _ = app.router.match("GET", "/api/users/profile")
        user_route, user_params, _ = app.router.match("GET", "/api/users/123")

        assert profile_route is not None
        assert user_route is not None
        assert profile_params == {}  # No parameters for specific route
        assert user_params == {"user_id": "123"}  # Parameter for dynamic route

    def test_optional_trailing_slash(self):
        """Test handling of trailing slashes"""
        app = App()

        @app.get("/api/data")
        def get_data(request):
            return JSONResponse({"data": "value"})

        # Test that both with and without trailing slash work
        # (This behavior depends on C implementation)
        route_without, _, _ = app.router.match("GET", "/api/data")
        route_with, _, _ = app.router.match("GET", "/api/data/")

        assert route_without is not None
        # Trailing slash behavior depends on C normalization

    def test_special_characters_in_params(self):
        """Test special characters in path parameters"""
        app = App()

        @app.get("/files/{filename}")
        def get_file(request):
            return JSONResponse({"filename": request.path_params.get("filename")})

        # Test that routes with special characters work
        route, params, _ = app.router.match("GET", "/files/document.pdf")
        assert route is not None
        assert params == {"filename": "document.pdf"}

        route, params, _ = app.router.match("GET", "/files/my-file_v2.txt")
        assert route is not None
        assert params == {"filename": "my-file_v2.txt"}

    def test_numeric_parameters(self):
        """Test numeric path parameters"""
        app = App()

        @app.get("/items/{item_id}")
        def get_item(request):
            # In a real handler, you might convert to int
            item_id = request.path_params.get("item_id")
            return JSONResponse({"item_id": item_id, "type": type(item_id).__name__})

        route, params, _ = app.router.match("GET", "/items/12345")
        assert route is not None
        assert params == {"item_id": "12345"}  # Parameters are always strings


class TestRouteValidation:
    """Test route validation and error conditions"""

    def test_duplicate_route_handling(self):
        """Test handling of duplicate routes"""
        app = App()

        @app.get("/api/test")
        def handler1(request):
            return JSONResponse({"version": 1})

        # Adding same route should generate warning
        with pytest.warns(UserWarning):
            @app.get("/api/test")
            def handler2(request):
                return JSONResponse({"version": 2})

    def test_route_with_no_parameters(self):
        """Test static routes without parameters"""
        app = App()

        @app.get("/static/route")
        def static_handler(request):
            return JSONResponse({"type": "static"})

        route, params, _ = app.router.match("GET", "/static/route")
        assert route is not None
        assert params == {}

    def test_root_route(self):
        """Test root route handling"""
        app = App()

        @app.get("/")
        def root_handler(request):
            return JSONResponse({"message": "root"})

        route, params, _ = app.router.match("GET", "/")
        assert route is not None
        assert params == {}

    def test_multiple_http_methods_same_path(self):
        """Test multiple HTTP methods on same path"""
        app = App()

        @app.get("/api/resource")
        def get_resource(request):
            return JSONResponse({"action": "get"})

        @app.post("/api/resource")
        def create_resource(request):
            return JSONResponse({"action": "create"})

        @app.put("/api/resource")
        def update_resource(request):
            return JSONResponse({"action": "update"})

        @app.delete("/api/resource")
        def delete_resource(request):
            return JSONResponse({"action": "delete"})

        routes = app.router.routes()
        assert len(routes) == 4

        methods = {r["method"] for r in routes}
        assert methods == {"GET", "POST", "PUT", "DELETE"}

        # Test that all methods are detected as allowed
        route, params, allowed = app.router.match("PATCH", "/api/resource")
        assert route is None  # PATCH not registered
        assert allowed == {"GET", "POST", "PUT", "DELETE"}


class TestIntegrationWithC:
    """Test integration points with C code"""

    def test_route_registration_with_c_backend(self):
        """Test that Python routes are properly registered with C backend"""
        app = App()

        @app.get("/test")
        def test_handler(request):
            return JSONResponse({"test": True})

        # Verify route is in Python router
        routes = app.router.routes()
        assert len(routes) == 1
        assert routes[0]["method"] == "GET"
        assert routes[0]["path"] == "/test"

    def test_path_parameter_format_compatibility(self):
        """Test that path parameter format is compatible with C implementation"""
        app = App()

        @app.get("/users/{user_id}/posts/{post_id}")
        def get_post(request):
            return JSONResponse({
                "user_id": request.path_params.get("user_id"),
                "post_id": request.path_params.get("post_id")
            })

        # Test that parameter names are extracted correctly
        routes = app.router.routes()
        route_info = routes[0]
        assert "{user_id}" in route_info["path"]
        assert "{post_id}" in route_info["path"]

    def test_method_normalization(self):
        """Test HTTP method normalization compatibility"""
        app = App()

        # Test that methods are normalized consistently
        def handler(request):
            return JSONResponse({})

        # These should all be treated as the same method
        app.router.add_route("GET", "/test1", handler)
        app.router.add_route("get", "/test2", handler)
        app.router.add_route("Get", "/test3", handler)

        routes = app.router.routes()
        methods = {r["method"] for r in routes}
        # All should be normalized to uppercase (or whatever the C code expects)
        assert len(methods) <= 2  # Should be normalized
